@@ -49,7 +49,7 @@ main (gint argc, gchar *argv[])
   NcDEDataClusterEntries de_data_cluster = NC_DE_DATA_CLUSTER_ENTRIES;
   NcDEFitEntries de_fit = NC_DE_FIT_ENTRIES;
   NcDistance *dist;
-  NcmMSet *mset, *fiduc;
+  NcmMSet *mset, *fiduc = NULL;
   GError *error = NULL;
   GOptionContext *context;
   GOptionEntry *de_model_entries = NULL;
@@ -59,6 +59,7 @@ main (gint argc, gchar *argv[])
   GPtrArray *ca_array = NULL;
   gchar *full_cmd_line = NULL;
   gchar *runconf_cmd_line = NULL;
+  gboolean is_de = FALSE;
 
   ncm_cfg_init ();
 
@@ -183,9 +184,13 @@ main (gint argc, gchar *argv[])
   ncm_message ("# Command Line: %s\n", full_cmd_line);
 
   dset = ncm_dataset_new ();
-  model = nc_hicosmo_new_from_name (NC_TYPE_HICOSMO_DE, de_model.model_name);
+  model = nc_hicosmo_new_from_name (NC_TYPE_HICOSMO, de_model.model_name);
   mset = ncm_mset_new (NCM_MODEL (model), NULL);
   dist = nc_distance_new (2.0);
+  ncm_cfg_set_named_instance (dist, "MainDist", FALSE);
+
+  if (g_type_is_a (G_OBJECT_TYPE (model), NC_TYPE_HICOSMO_DE))
+    is_de = TRUE;
 
   if (de_model.help_names)
   {
@@ -200,19 +205,25 @@ main (gint argc, gchar *argv[])
 
   if (de_model.flat)
   {
+    if (!is_de)
+      g_error ("flat option is valid only for darkenergy models");
     nc_hicosmo_de_omega_x2omega_k (model);
     ncm_model_param_set (NCM_MODEL (model), NC_HICOSMO_DE_OMEGA_X, 0.0);
-    ncm_mset_param_set_ftype (mset, NC_HICOSMO_ID, NC_HICOSMO_DE_OMEGA_X, NCM_PARAM_TYPE_FIXED);
+    ncm_mset_param_set_ftype (mset, nc_hicosmo_id (), NC_HICOSMO_DE_OMEGA_X, NCM_PARAM_TYPE_FIXED);
   }
   else if (de_model.Omega_k)
   {
+    if (!is_de)
+      g_error ("omegak option is valid only for darkenergy models");
     nc_hicosmo_de_omega_x2omega_k (model);
-    ncm_mset_param_set_ftype (mset, NC_HICOSMO_ID, NC_HICOSMO_DE_OMEGA_X, NCM_PARAM_TYPE_FREE);
+    ncm_mset_param_set_ftype (mset, nc_hicosmo_id (), NC_HICOSMO_DE_OMEGA_X, NCM_PARAM_TYPE_FREE);
   }
 
   if (de_model.pos_Omega_x)
   {
-    ncm_prior_add_positive (lh, NC_HICOSMO_ID, NC_HICOSMO_DE_OMEGA_X);
+    if (!is_de)
+      g_error ("omegak > 0 option is valid only for darkenergy models");
+    ncm_prior_add_positive (lh, nc_hicosmo_id (), NC_HICOSMO_DE_OMEGA_X);
   }
   
   if (de_data_simple.snia_id != NULL)
@@ -232,14 +243,15 @@ main (gint argc, gchar *argv[])
     }
     else if (snia_id->value >= NC_DATA_SNIA_COV_START && snia_id->value <= NC_DATA_SNIA_COV_END)
     {
-      NcSNIADistCov *dcov = nc_snia_dist_cov_new (dist, 0);
+      NcSNIADistCov *dcov = nc_snia_dist_cov_new (dist);
       NcmData *data;
 
       if (de_data_simple.snia_prop != NULL)
         ncm_cfg_object_set_property (G_OBJECT (dcov), de_data_simple.snia_prop);
 
-      nc_data_snia_load_cat (dcov, snia_id->value);
-      data = nc_data_snia_cov_new (dcov, de_data_simple.snia_use_det);
+      data = nc_data_snia_cov_new (de_data_simple.snia_use_det);
+      nc_data_snia_load_cat (NC_DATA_SNIA_COV (data), snia_id->value);
+      
       ncm_mset_set (mset, NCM_MODEL (dcov));
       ncm_dataset_append_data (dset, data);
       ncm_data_free (data);
@@ -265,30 +277,61 @@ main (gint argc, gchar *argv[])
 
   if (de_data_simple.bao_id != NULL)
   {
-    const GEnumValue *bao_id = ncm_cfg_get_enum_by_id_name_nick (NC_TYPE_DATA_BAO_ID,
-                                                                 de_data_simple.bao_id);
-    if (bao_id != NULL)
+    guint i;
+    guint nbao = g_strv_length (de_data_simple.bao_id);
+    for (i = 0; i < nbao; i++)
     {
-      NcmData *bao_data = nc_data_bao_create (dist, bao_id->value);
-      ncm_dataset_append_data (dset, bao_data);
-      ncm_data_free (bao_data);
+      gchar *bao_id_i = de_data_simple.bao_id[i];
+      const GEnumValue *bao_id = ncm_cfg_get_enum_by_id_name_nick (NC_TYPE_DATA_BAO_ID, bao_id_i);
+      if (bao_id != NULL)
+      {
+        NcmData *bao_data = nc_data_bao_create (dist, bao_id->value);
+        ncm_dataset_append_data (dset, bao_data);
+        ncm_data_free (bao_data);
+      }
+      else
+        g_error ("BAO sample '%s' not found run --bao-list to list the available options", bao_id_i);
     }
-    else
-      g_error ("BAO sample '%s' not found run --bao-list to list the available options", de_data_simple.bao_id);
   }
 
   if (de_data_simple.H_id != NULL)
   {
-    const GEnumValue *H_id = ncm_cfg_get_enum_by_id_name_nick (NC_TYPE_DATA_HUBBLE_ID,
-                                                               de_data_simple.H_id);
-    if (H_id != NULL)
+    guint i;
+    guint nHz = g_strv_length (de_data_simple.H_id);
+    
+    for (i = 0; i < nHz; i++)
     {
-      NcmData *H_data = nc_data_hubble_new (H_id->value);
-      ncm_dataset_append_data (dset, H_data);
-      ncm_data_free (H_data);
+      gchar *H_id_i = de_data_simple.H_id[i];
+      const GEnumValue *H_id = ncm_cfg_get_enum_by_id_name_nick (NC_TYPE_DATA_HUBBLE_ID, H_id_i);
+      if (H_id != NULL)
+      {
+        NcmData *H_data = nc_data_hubble_new (H_id->value);
+        ncm_dataset_append_data (dset, H_data);
+        ncm_data_free (H_data);
+      }
+      else
+        g_error ("Hubble sample '%s' not found run --H-list to list the available options", H_id_i);
+    }
+  }
+
+  if (de_data_simple.H_BAO_id != NULL)
+  {
+    guint i;
+    guint nHrs = g_strv_length (de_data_simple.H_BAO_id);
+    
+    for (i = 0; i < nHrs; i++)
+    {
+      gchar *Hrs_id_i = de_data_simple.H_BAO_id[i];
+      const GEnumValue *Hrs_id = ncm_cfg_get_enum_by_id_name_nick (NC_TYPE_DATA_HUBBLE_BAO_ID, Hrs_id_i);
+      if (Hrs_id != NULL)
+      {
+        NcmData *Hrs_data = nc_data_hubble_bao_new (dist, Hrs_id->value);
+        ncm_dataset_append_data (dset, Hrs_data);
+        ncm_data_free (Hrs_data);
+      }
+      else
+        g_error ("Hubble BAO sample '%s' not found run --H-BAO-list to list the available options", Hrs_id_i);
     }
-    else
-      g_error ("Hubble sample '%s' not found run --hubble-list to list the available options", de_data_simple.H_id);
   }
 
   if (de_data_simple.cluster_id != NULL)
@@ -306,15 +349,37 @@ main (gint argc, gchar *argv[])
   if (de_data_simple.BBN)
     nc_hicosmo_de_new_add_bbn (lh);
 
+  if (de_data_simple.BBN_Ob)
+  {
+    NcmMSetFunc *Omega_bh2 = nc_hicosmo_create_mset_func0 (&nc_hicosmo_Omega_bh2);
+    ncm_prior_add_gaussian_const_func (lh, Omega_bh2, 0.022, 0.002);
+    ncm_mset_func_free (Omega_bh2);
+  }
+
   if (de_data_simple.H0_Hst)
   {
-    if (ncm_mset_param_get_ftype (mset, NC_HICOSMO_ID, NC_HICOSMO_DE_H0) == NCM_PARAM_TYPE_FIXED)
+    if (ncm_mset_param_get_ftype (mset, nc_hicosmo_id (), NC_HICOSMO_DE_H0) == NCM_PARAM_TYPE_FIXED)
       g_warning ("A prior on H0 was required but H0 was kept fixed. This prior will be useless.");
 
-    ncm_prior_add_gaussian_data (lh, NC_HICOSMO_ID, NC_HICOSMO_DE_H0, 73.8, 2.4);
+    ncm_prior_add_gaussian_data (lh, nc_hicosmo_id (), NC_HICOSMO_DE_H0, 73.8, 2.4);
   }
-  
-  fiduc = ncm_mset_dup (mset);
+
+  if (de_fit.qspline_cp && TRUE)
+  {
+    if (!NC_IS_HICOSMO_QSPLINE (model))
+      g_error ("Continuity priors are only valid for NcHICosmoQSPline model");
+    else
+    {
+      NcHICosmoQSplineContPrior *qspline_cp = 
+        nc_hicosmo_qspline_add_continuity_priors (NC_HICOSMO_QSPLINE (model), lh, de_fit.qspline_cp_sigma);
+      nc_hicosmo_qspline_cont_prior_free (qspline_cp);
+    }
+  }
+
+  if (de_fit.save_mset != NULL)
+    ncm_mset_save (mset, de_fit.save_mset, TRUE);
+  if (de_fit.fiducial != NULL)
+    fiduc = ncm_mset_load (de_fit.fiducial); 
 
   {
     const GEnumValue *fit_type_id = ncm_cfg_get_enum_by_id_name_nick (NCM_TYPE_FIT_TYPE, de_fit.fit_type);
@@ -327,6 +392,19 @@ main (gint argc, gchar *argv[])
     fit = ncm_fit_new (fit_type_id->value, de_fit.fit_algo, lh, mset, fit_diff_id->value);
   }
 
+  if (de_fit.qspline_cp && FALSE)
+  {
+    if (!NC_IS_HICOSMO_QSPLINE (model))
+      g_error ("Continuity priors are only valid for NcHICosmoQSPline model");
+    else
+    {
+      NcHICosmoQSplineContPrior *qspline_cp = 
+        nc_hicosmo_qspline_add_continuity_constraints (NC_HICOSMO_QSPLINE (model), fit, de_fit.qspline_cp_sigma);
+      nc_hicosmo_qspline_cont_prior_free (qspline_cp);
+    }
+  }
+
+  
   de_fit.fisher = (de_fit.fisher || (de_fit.nsigma_fisher != -1) || (de_fit.nsigma != -1) || (de_fit.onedim_cr != NULL));
   de_fit.fit = (de_fit.fit || de_fit.fisher);
   de_fit.save_best_fit = (de_fit.save_best_fit || de_fit.save_fisher);
@@ -334,6 +412,7 @@ main (gint argc, gchar *argv[])
   if (de_fit.fit)
   {
     fit->params_reltol = 1e-5;
+    ncm_fit_set_maxiter (fit, de_fit.max_iter);
     ncm_fit_run (fit, de_fit.msg_level);
     
     ncm_fit_log_info (fit);
@@ -381,14 +460,11 @@ main (gint argc, gchar *argv[])
     NcmFitMC *mc = ncm_fit_mc_new (fit);
     gdouble m2lnL = fit->fstate->m2lnL;
 
-    if (de_fit.fiducial)
+    if (de_fit.fiducial != NULL)
       resample_mset = fiduc;
     else
       resample_mset = fit->mset;
 
-    ncm_dataset_log_info (fit->lh->dset);
-    ncm_mset_pretty_log (resample_mset);
-
     ncm_fit_mc_run (mc, resample_mset, de_fit.mc_ni, de_fit.montecarlo, de_fit.msg_level);
     ncm_fit_mc_mean_covar (mc);
     ncm_fit_mc_gof_pdf (mc);
@@ -426,12 +502,12 @@ main (gint argc, gchar *argv[])
           break;
       }
       {
-        NcmMSetPIndex pi = {NC_HICOSMO_ID, p_n};
+        NcmMSetPIndex pi = {nc_hicosmo_id (), p_n};
         NcmLHRatio1d *lhr1d = ncm_lh_ratio1d_new (fit, &pi);
         ncm_lh_ratio1d_find_bounds (lhr1d, fit->mtype, prob_sigma, &err_inf, &err_sup);
         ncm_lh_ratio1d_free (lhr1d);
       }
-      //ncm_fit_cr_1dim (fit, NC_HICOSMO_ID, p_n, prob_sigma, 1, &err_inf, &err_sup);
+      //ncm_fit_cr_1dim (fit, nc_hicosmo_id (), p_n, prob_sigma, 1, &err_inf, &err_sup);
       ncm_message ("#  One dimension confidence region for %s[%02d] = % .5g (% .5g, % .5g)\n",
                    ncm_model_param_name (NCM_MODEL (model), p_n), p_n,
                    ncm_model_param_get (NCM_MODEL (model), p_n), err_inf, err_sup);
@@ -457,8 +533,8 @@ main (gint argc, gchar *argv[])
     NcmLHRatio2dRegion *rg_2sigma = NULL;
     NcmLHRatio2dRegion *rg_3sigma = NULL;
 
-    pi1.mid = NC_HICOSMO_ID;
-    pi2.mid = NC_HICOSMO_ID;
+    pi1.mid = nc_hicosmo_id ();
+    pi2.mid = nc_hicosmo_id ();
     pi1.pid = de_fit.bidim_cr[0];
     pi2.pid = de_fit.bidim_cr[1];
 
@@ -528,8 +604,8 @@ main (gint argc, gchar *argv[])
     NcmLHRatio2dRegion *rg_2sigma = NULL;
     NcmLHRatio2dRegion *rg_3sigma = NULL;
 
-    pi1.mid = NC_HICOSMO_ID;
-    pi2.mid = NC_HICOSMO_ID;
+    pi1.mid = nc_hicosmo_id ();
+    pi2.mid = nc_hicosmo_id ();
     pi1.pid = de_fit.bidim_cr[0];
     pi2.pid = de_fit.bidim_cr[1];
 
@@ -610,6 +686,39 @@ main (gint argc, gchar *argv[])
     g_free (mfile);
   }
 
+  if (de_fit.kinematics_sigma)
+  {
+    NcmMSetFunc *dec_param_func = nc_hicosmo_create_mset_func1 (nc_hicosmo_q);
+    NcmMSetFunc *E2_func = nc_hicosmo_create_mset_func1 (nc_hicosmo_E2);
+    NcmMSetFunc *Em2_func = nc_hicosmo_create_mset_func1 (nc_hicosmo_Em2);
+    NcmMSetFunc *dec_func = nc_hicosmo_create_mset_func1 (nc_hicosmo_dec);
+    NcmMSetFunc *wec_func = nc_hicosmo_create_mset_func1 (nc_hicosmo_wec);
+    gint i;
+
+    ncm_message ("# Kinematics data:\n");
+    for (i = 0; i < de_fit.kinematics_n; i++)
+    {
+      gdouble z = de_fit.kinematics_z / (de_fit.kinematics_n - 1.0) * i;
+      gdouble q_z, E2_z, Em2_z, dec, wec;
+      gdouble sigma_q_z, sigma_E2_z, sigma_Em2_z, sigma_dec, sigma_wec;
+      ncm_fit_function_error (fit, dec_param_func, &z, FALSE, &q_z, &sigma_q_z);
+      ncm_fit_function_error (fit, E2_func, &z, FALSE, &E2_z, &sigma_E2_z);
+      ncm_fit_function_error (fit, Em2_func, &z, FALSE, &Em2_z, &sigma_Em2_z);
+      ncm_fit_function_error (fit, dec_func, &z, FALSE, &dec, &sigma_dec);
+      ncm_fit_function_error (fit, wec_func, &z, FALSE, &wec, &sigma_wec);
+      ncm_message ("% 20.15g % 20.15g % 20.15g % 20.15g % 20.15g % 20.15g % 20.15g % 20.15g % 20.15g % 20.15g % 20.15g\n", 
+                   z, 
+                   q_z, sigma_q_z, 
+                   E2_z, sigma_E2_z, 
+                   Em2_z, sigma_Em2_z,
+                   dec, sigma_dec,
+                   wec, sigma_wec);
+    }
+    ncm_mset_func_free (dec_param_func);
+    ncm_mset_func_free (E2_func);
+    ncm_mset_func_free (Em2_func);
+  }
+
   if (ca_array != NULL)
   {
     g_ptr_array_free (ca_array, TRUE);
@@ -619,9 +728,12 @@ main (gint argc, gchar *argv[])
   g_free (de_data_simple_entries); de_data_simple_entries = NULL;
   g_free (de_data_cluster_entries); de_data_cluster_entries = NULL;
   g_free (de_fit_entries); de_fit_entries = NULL;
+
+  if (fiduc != NULL)
+    ncm_mset_free (fiduc);
   
+  ncm_cfg_free_named_instances ();
   ncm_model_free (NCM_MODEL (model));
-  ncm_mset_free (fiduc);
   ncm_mset_free (mset);
   ncm_fit_free (fit);
   ncm_likelihood_free (lh);
