@@ -29,6 +29,7 @@
 
 #include <glib.h>
 #include <glib-object.h>
+#include <numcosmo/build_cfg.h>
 #include <numcosmo/math/ncm_model.h>
 #include <numcosmo/nc_hicosmo.h>
 
@@ -84,10 +85,10 @@ struct _NcClusterMass
   NcmModel parent_instance;
 };
 
-extern gint32 NC_CLUSTER_MASS_ID;
-
 GType nc_cluster_mass_get_type (void) G_GNUC_CONST;
 
+NCM_MSET_MODEL_DECLARE_ID (nc_cluster_mass);
+
 NcClusterMass *nc_cluster_mass_new_from_name (gchar *mass_name);
 NcClusterMass *nc_cluster_mass_ref (NcClusterMass *clusterm);
 void nc_cluster_mass_free (NcClusterMass *clusterm);
