@@ -28,7 +28,7 @@
 
 #include <glib.h>
 #include <glib-object.h>
-
+#include <numcosmo/build_cfg.h>
 #include <gsl/gsl_const_num.h>
 #include <math.h>
 
@@ -63,6 +63,7 @@ GType ncm_c_get_type (void) G_GNUC_CONST;
  *******************************************************************************/
 
 G_INLINE_FUNC long double ncm_c_sqrt_1_4pi (void) G_GNUC_CONST;
+G_INLINE_FUNC long double ncm_c_sqrt_2pi (void) G_GNUC_CONST;
 G_INLINE_FUNC long double ncm_c_sqrt_3_4pi (void) G_GNUC_CONST;
 G_INLINE_FUNC long double ncm_c_lnpi_4 (void) G_GNUC_CONST;
 G_INLINE_FUNC long double ncm_c_ln2pi (void) G_GNUC_CONST;
@@ -227,6 +228,9 @@ G_BEGIN_DECLS
 G_INLINE_FUNC long double ncm_c_sqrt_1_4pi (void)
 { return 0.28209479177387814347403972578038630L; }
 
+G_INLINE_FUNC long double ncm_c_sqrt_2pi (void)
+{ return 2.5066282746310005024157652848110452L; }
+
 G_INLINE_FUNC long double ncm_c_sqrt_3_4pi (void)
 { return 0.48860251190291992158638462283834700L; }
 
