@@ -54,10 +54,12 @@
 #include <gsl/gsl_cdf.h>
 #include <gsl/gsl_linalg.h>
 #include <gsl/gsl_randist.h>
+#include <gsl/gsl_machine.h>
 
 enum
 {
   PROP_0,
+  PROP_MAXITER,
   PROP_LIKELIHOOD,
   PROP_MSET,
   PROP_STATE,
@@ -66,15 +68,71 @@ enum
 };
 
 G_DEFINE_ABSTRACT_TYPE (NcmFit, ncm_fit, G_TYPE_OBJECT);
+G_DEFINE_BOXED_TYPE (NcmFitConstraint, ncm_fit_constraint, (GBoxedCopyFunc)&ncm_fit_constraint_dup, (GBoxedFreeFunc)&ncm_fit_constraint_free);
+
+/**
+ * ncm_fit_constraint_new:
+ * @fit: FIXME
+ * @func: FIXME
+ * @tot: FIXME
+ * 
+ * FIXME
+ * 
+ * Returns: (transfer full): FIXME
+ */
+NcmFitConstraint *
+ncm_fit_constraint_new (NcmFit *fit, NcmMSetFunc *func, gdouble tot)
+{
+  NcmFitConstraint *fitc = g_new (NcmFitConstraint, 1);
+  g_assert (ncm_mset_func_is_scalar (func) && ncm_mset_func_is_const (func));
+  fitc->fit = ncm_fit_ref (fit);
+  fitc->func = ncm_mset_func_ref (func);
+  fitc->tot = tot;
+  return fitc;
+}
+
+/**
+ * ncm_fit_constraint_dup:
+ * @fitc: FIXME
+ * 
+ * FIXME
+ * 
+ * Returns: (transfer full): FIXME
+ */
+NcmFitConstraint *
+ncm_fit_constraint_dup (NcmFitConstraint *fitc)
+{
+  return ncm_fit_constraint_new (fitc->fit, fitc->func, fitc->tot);
+}
+
+/**
+ * ncm_fit_constraint_free:
+ * @fitc: FIXME
+ * 
+ * FIXME
+ * 
+ */
+void
+ncm_fit_constraint_free (NcmFitConstraint *fitc)
+{
+  ncm_mset_func_free (fitc->func);
+  g_free (fitc);
+}
 
 static void
 ncm_fit_init (NcmFit *fit)
 {
-  fit->maxeval       = NCM_FIT_MAXEVAL;
+  fit->maxiter       = NCM_FIT_MAXITER;
   fit->m2lnL_reltol  = NCM_FIT_DEFAULT_M2LNL_RELTOL * 0.0 + 1e-8;
   fit->params_reltol = NC_HICOSMO_DEFAULT_PARAMS_RELTOL * 0.0 + 1e-8;
   fit->timer = g_timer_new ();
   fit->mtype = NCM_FIT_RUN_MSGS_NONE;
+
+  fit->equality_constraints = g_ptr_array_sized_new (10);
+  g_ptr_array_set_free_func (fit->equality_constraints, (GDestroyNotify) &ncm_fit_constraint_free);
+
+  fit->inequality_constraints = g_ptr_array_sized_new (10);
+  g_ptr_array_set_free_func (fit->inequality_constraints, (GDestroyNotify) &ncm_fit_constraint_free);
 }
 
 static void
@@ -121,6 +179,9 @@ _ncm_fit_set_property (GObject *object, guint prop_id, const GValue *value, GPar
 
   switch (prop_id)
   {
+    case PROP_MAXITER:
+      ncm_fit_set_maxiter (fit, g_value_get_uint (value));
+      break;
     case PROP_LIKELIHOOD:
       ncm_likelihood_clear (&fit->lh);
       fit->lh = g_value_dup_object (value);
@@ -146,6 +207,9 @@ _ncm_fit_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec
 
   switch (prop_id)
   {
+    case PROP_MAXITER:
+      g_value_set_uint (value, ncm_fit_get_maxiter (fit));
+      break;
     case PROP_LIKELIHOOD:
       g_value_set_object (value, fit->lh);
       break;
@@ -170,6 +234,18 @@ ncm_fit_dispose (GObject *object)
   ncm_mset_clear (&fit->mset);
   ncm_fit_state_clear (&fit->fstate);
 
+  if (fit->equality_constraints != NULL)
+  {
+    g_ptr_array_unref (fit->equality_constraints);
+    fit->equality_constraints = NULL;
+  }
+  
+  if (fit->inequality_constraints != NULL)
+  {
+    g_ptr_array_unref (fit->inequality_constraints);
+    fit->inequality_constraints = NULL;
+  }
+
   /* Chain up : end */
   G_OBJECT_CLASS (ncm_fit_parent_class)->dispose (object);
 }
@@ -201,6 +277,13 @@ ncm_fit_class_init (NcmFitClass *klass)
   klass->is_least_squares = FALSE;
   klass->reset            = &_ncm_fit_reset;
 
+  g_object_class_install_property (object_class,
+                                   PROP_MAXITER,
+                                   g_param_spec_uint ("maxiter",
+                                                      NULL,
+                                                      "Maximum number of interations",
+                                                      0, G_MAXUINT32, NCM_FIT_MAXITER,
+                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
   g_object_class_install_property (object_class,
                                    PROP_LIKELIHOOD,
                                    g_param_spec_object ("likelihood",
@@ -445,6 +528,119 @@ ncm_fit_ls_set_state (NcmFit *fit, gdouble prec, NcmVector *x, NcmVector *f, Ncm
   ncm_matrix_memcpy (fit->fstate->ls_J, J);
 }
 
+/**
+ * ncm_fit_set_maxiter:
+ * @fit: a #NcmFit.
+ * @maxiter: FIXME.
+ *
+ * FIXME
+ */
+void 
+ncm_fit_set_maxiter (NcmFit *fit, guint maxiter)
+{
+  fit->maxiter = maxiter;
+}
+
+/**
+ * ncm_fit_get_maxiter:
+ * @fit: a #NcmFit.
+ *
+ * FIXME
+ * 
+ * Returns: FIXME
+ */
+guint 
+ncm_fit_get_maxiter (NcmFit *fit)
+{
+  return fit->maxiter;
+}
+
+/**
+ * ncm_fit_add_equality_constraint:
+ * @fit: a #NcmFit.
+ * @func: FIXME
+ * @tot: FIXME
+ *
+ * FIXME
+ * 
+ */
+void 
+ncm_fit_add_equality_constraint (NcmFit *fit, NcmMSetFunc *func, gdouble tot)
+{
+  NcmFitConstraint *fitc = ncm_fit_constraint_new (fit, func, tot);
+  g_ptr_array_add (fit->equality_constraints, fitc);
+}
+
+/**
+ * ncm_fit_add_inequality_constraint:
+ * @fit: a #NcmFit.
+ * @func: FIXME
+ * @tot: FIXME
+ *
+ * FIXME
+ * 
+ */
+void
+ncm_fit_add_inequality_constraint (NcmFit *fit, NcmMSetFunc *func, gdouble tot)
+{
+  NcmFitConstraint *fitc = ncm_fit_constraint_new (fit, func, tot);
+  g_ptr_array_add (fit->inequality_constraints, fitc);
+}
+
+/**
+ * ncm_fit_remove_equality_constraints:
+ * @fit: a #NcmFit.
+ *
+ * FIXME
+ * 
+ */
+void 
+ncm_fit_remove_equality_constraints (NcmFit *fit)
+{
+  g_ptr_array_set_size (fit->equality_constraints, 0);
+}
+
+/**
+ * ncm_fit_remove_inequality_constraints:
+ * @fit: a #NcmFit.
+ *
+ * FIXME
+ * 
+ */
+void 
+ncm_fit_remove_inequality_constraints (NcmFit *fit)
+{
+  g_ptr_array_set_size (fit->inequality_constraints, 0);
+}
+
+/**
+ * ncm_fit_has_equality_constraints:
+ * @fit: a #NcmFit.
+ *
+ * FIXME
+ * 
+ * Returns: FIXME
+ */
+guint 
+ncm_fit_has_equality_constraints (NcmFit *fit)
+{
+  return fit->equality_constraints->len;
+}
+
+/**
+ * ncm_fit_has_inequality_constraints:
+ * @fit: a #NcmFit.
+ *
+ * FIXME
+ * 
+ * Returns: FIXME
+ */
+guint
+ncm_fit_has_inequality_constraints (NcmFit *fit)
+{
+  return fit->inequality_constraints->len;
+}
+
 /**
  * ncm_fit_ls_covar:
  * @fit: a #NcmFit.
@@ -1080,11 +1276,12 @@ ncm_fit_m2lnL_grad_nd_ce (NcmFit *fit, NcmVector *grad)
   for (i = 0; i < fparam_len; i++)
   {
     const gdouble p = ncm_mset_fparam_get (fit->mset, i);
-    const gdouble p_scale = ncm_mset_fparam_get_scale (fit->mset, i);
-    const gdouble h = (p_scale * NCM_FIT_NUMDIFF_SCALE);
+    const gdouble p_scale = GSL_MAX (fabs (p), ncm_mset_fparam_get_scale (fit->mset, i));
+    const gdouble h = p_scale * GSL_ROOT3_DBL_EPSILON;
     const gdouble pph = p + h;
     const gdouble pmh = p - h;
-    const gdouble one_2h = 1.0 / (2.0 * h);
+    const gdouble twoh = pph - pmh;
+    const gdouble one_2h = 1.0 / twoh;
     gdouble m2lnL_pph, m2lnL_pmh;
 
     ncm_mset_fparam_set (fit->mset, i, pph);
@@ -1200,9 +1397,10 @@ ncm_fit_m2lnL_val_grad_nd_fo (NcmFit *fit, gdouble *m2lnL, NcmVector *grad)
   for (i = 0; i < fparam_len; i++)
   {
     const gdouble p = ncm_mset_fparam_get (fit->mset, i);
-    const gdouble p_scale = ncm_mset_fparam_get_scale (fit->mset, i);
-    const gdouble h = (p_scale * NCM_FIT_NUMDIFF_SCALE);
-    const gdouble pph = p + h;
+    const gdouble p_scale = GSL_MAX (fabs (p), ncm_mset_fparam_get_scale (fit->mset, i));
+    const gdouble htilde = GSL_SQRT_DBL_EPSILON * p_scale;
+    const gdouble pph = p + htilde;
+    const gdouble h = pph - p;
     const gdouble one_h = 1.0 / h;
     gdouble m2lnL_pph;
     ncm_mset_fparam_set (fit->mset, i, pph);
@@ -1293,9 +1491,10 @@ ncm_fit_ls_J_nd_fo (NcmFit *fit, NcmMatrix *J)
   {
     NcmVector *J_col_i = ncm_matrix_get_col (J, i);
     const gdouble p = ncm_mset_fparam_get (fit->mset, i);
-    const gdouble p_scale = ncm_mset_fparam_get_scale (fit->mset, i);
-    const gdouble h = (p_scale * NCM_FIT_NUMDIFF_SCALE);
-    const gdouble pph = p + h;
+    const gdouble p_scale = GSL_MAX (fabs (p), ncm_mset_fparam_get_scale (fit->mset, i));
+    const gdouble htilde = p_scale * GSL_SQRT_DBL_EPSILON;
+    const gdouble pph = p + htilde;
+    const gdouble h = pph - p;
     const gdouble one_h = 1.0 / h;
     gint k;
     ncm_mset_fparam_set (fit->mset, i, pph);
@@ -1328,11 +1527,12 @@ ncm_fit_ls_J_nd_ce (NcmFit *fit, NcmMatrix *J)
   {
     NcmVector *J_col_i = ncm_matrix_get_col (J, i);
     const gdouble p = ncm_mset_fparam_get (fit->mset, i);
-    const gdouble p_scale = ncm_mset_fparam_get_scale (fit->mset, i);
-    const gdouble h = (p_scale * NCM_FIT_NUMDIFF_SCALE);
+    const gdouble p_scale = GSL_MAX (fabs (p), ncm_mset_fparam_get_scale (fit->mset, i));
+    const gdouble h = p_scale * GSL_ROOT3_DBL_EPSILON;
     const gdouble pph = p + h;
     const gdouble pmh = p - h;
-    const gdouble one_2h = 1.0 / (2.0 * h);
+    const gdouble twoh = pph - pmh;
+    const gdouble one_2h = 1.0 / twoh;
     gint k;
 
     ncm_mset_fparam_set (fit->mset, i, pph);
@@ -1796,40 +1996,36 @@ ncm_fit_lr_test (NcmFit *fit, NcmModelID mid, guint pid, gdouble val, gint dof)
  * ncm_fit_function_error:
  * @fit: a #NcmFit
  * @func: a #NcmMSetFunc
- * @z: FIXME
+ * @x: FIXME
  * @pretty_print: FIXME
+ * @f: (out): FIXME
+ * @sigma_f: (out): FIXME
  *
  * FIXME
  *
- * Returns: FIXME
  */
-gdouble
-ncm_fit_function_error (NcmFit *fit, NcmMSetFunc *func, gdouble z, gboolean pretty_print)
+void
+ncm_fit_function_error (NcmFit *fit, NcmMSetFunc *func, gdouble *x, gboolean pretty_print, gdouble *f, gdouble *sigma_f)
 {
-  g_assert_not_reached ();
-  /*
-   gdouble result;
-   gint ret;
-   guint fparam_len = ncm_mset_fparam_len (fit->mset);
-   NcmVector *v = ncm_vector_new (free_params_len);
+  NcmVector *v = ncm_mset_func_numdiff_fparams (func, fit->mset, x, NULL);
+  NcmVector *tmp1 = ncm_vector_dup (v);
+  gdouble result;
+  gint ret;
 
-   ncm_mset_func_numdiff_fparams (func, );
-   NCM_FUNC_DF (func, fit->mset, fit->pt, z, fit->df);
+  ret = gsl_blas_dgemv (CblasNoTrans, 1.0, NCM_MATRIX_GSL (fit->fstate->covar), ncm_vector_gsl (v), 0.0, ncm_vector_gsl (tmp1));
+  NCM_TEST_GSL_RESULT("ncm_fit_function_error[covar.v]", ret);
+  ret = gsl_blas_ddot (ncm_vector_gsl (v), ncm_vector_gsl (tmp1), &result);
+  NCM_TEST_GSL_RESULT("ncm_fit_function_error[v.covar.v]", ret);
 
-   ret = gsl_blas_dgemv (CblasNoTrans, 1.0, NCM_MATRIX_GSL (fit->fstate->covar), ncm_vector_gsl (fit->df), 0.0, ncm_vector_gsl (tmp1));
-   NCM_TEST_GSL_RESULT("ncm_fit_function_error[covar.v]", ret);
-   ret = gsl_blas_ddot (ncm_vector_gsl (fit->df), ncm_vector_gsl (tmp1), &result);
-   NCM_TEST_GSL_RESULT("ncm_fit_function_error[v.covar.v]", ret);
+  *f = ncm_mset_func_eval (func, fit->mset, x);
+  *sigma_f = sqrt(result);
+  if (pretty_print)
+    g_message ("# % -12.4g +/- % -12.4g\n", *f, *sigma_f);
 
-   if (pretty_print)
-   g_message ("# % -12.4g +/- % -12.4g\n", NCM_FUNC_F(func, fit->mset, z), sqrt(result));
+  ncm_vector_free (v);
+  ncm_vector_free (tmp1);
 
-   ncm_fit_params_return_tmp_vector (fit->pt, tmp1);
-   
-   ncm_vector_free (v);
-
-   return sqrt(result);
-   */
+  return;
 }
 
 /**
