@@ -1,5 +1,5 @@
 /***************************************************************************
- *            fit.h
+ *            ncm_fit.h
  *
  *  Fri Aug 15 15:26:31 2008
  *  Copyright  2008  Sandro Dias Pinto Vitenti
@@ -28,6 +28,7 @@
 
 #include <glib.h>
 #include <glib-object.h>
+#include <numcosmo/build_cfg.h>
 #include <numcosmo/math/ncm_vector.h>
 #include <numcosmo/math/ncm_matrix.h>
 #include <numcosmo/math/ncm_model.h>
@@ -52,8 +53,13 @@ G_BEGIN_DECLS
 #define NCM_IS_FIT_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), NCM_TYPE_FIT))
 #define NCM_FIT_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), NCM_TYPE_FIT, NcmFitClass))
 
+#define NCM_TYPE_FIT_CONSTRAINT    (ncm_fit_constraint_get_type ())
+#define NCM_FIT_CONSTRAINT(obj)    (G_TYPE_CHECK_INSTANCE_CAST ((obj), NCM_TYPE_FIT_CONSTRAINT, NcmFitConstraint))
+#define NCM_IS_FIT_CONSTRAINT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NCM_TYPE_FIT_CONSTRAINT))
+
 typedef struct _NcmFitClass NcmFitClass;
 typedef struct _NcmFit NcmFit;
+typedef struct _NcmFitConstraint NcmFitConstraint;
 
 /**
  * NcmFitType:
@@ -96,7 +102,6 @@ typedef void (*_NcmFitLSFJ) (NcmFit *fit, NcmVector *f, NcmMatrix *J);
 typedef void (*_NcmFitM2lnLGrad) (NcmFit *fit, NcmVector *grad);
 typedef void (*_NcmFitM2lnLValGrad) (NcmFit *fit, gdouble *m2lnL, NcmVector *grad);
 
-
 /**
  * NcmFitGrad:
  *
@@ -148,13 +153,28 @@ struct _NcmFit
   NcmFitState *fstate;
   NcmFitRunMsgs mtype;
   NcmFitGrad grad;
-  guint maxeval;
+  guint maxiter;
   gdouble m2lnL_reltol;
   gdouble params_reltol;
   GTimer *timer;
+  GPtrArray *equality_constraints;
+  GPtrArray *inequality_constraints;
+};
+
+struct _NcmFitConstraint
+{
+  /*< private >*/
+  NcmFit *fit;
+  NcmMSetFunc *func;
+  gdouble tot;
 };
 
 GType ncm_fit_get_type (void) G_GNUC_CONST;
+GType ncm_fit_constraint_get_type (void) G_GNUC_CONST;
+
+NcmFitConstraint *ncm_fit_constraint_new (NcmFit *fit, NcmMSetFunc *func, gdouble tot);
+NcmFitConstraint *ncm_fit_constraint_dup (NcmFitConstraint *fitc);
+void ncm_fit_constraint_free (NcmFitConstraint *fitc);
 
 NcmFit *ncm_fit_new (NcmFitType ftype, gchar *algo_name, NcmLikelihood *lh, NcmMSet *mset, NcmFitGradType gtype);
 NcmFit *ncm_fit_ref (NcmFit *fit);
@@ -164,6 +184,15 @@ void ncm_fit_clear (NcmFit **fit);
 
 void ncm_fit_set_grad_type (NcmFit *fit, NcmFitGradType gtype);
 void ncm_fit_ls_set_state (NcmFit *fit, gdouble prec, NcmVector *x, NcmVector *f, NcmMatrix *J);
+void ncm_fit_set_maxiter (NcmFit *fit, guint maxiter);
+guint ncm_fit_get_maxiter (NcmFit *fit);
+
+void ncm_fit_add_equality_constraint (NcmFit *fit, NcmMSetFunc *func, gdouble tot);
+void ncm_fit_add_inequality_constraint (NcmFit *fit, NcmMSetFunc *func, gdouble tot);
+void ncm_fit_remove_equality_constraints (NcmFit *fit);
+void ncm_fit_remove_inequality_constraints (NcmFit *fit);
+guint ncm_fit_has_equality_constraints (NcmFit *fit);
+guint ncm_fit_has_inequality_constraints (NcmFit *fit);
 
 gboolean ncm_fit_is_least_squares (NcmFit *fit);
 
@@ -230,14 +259,14 @@ void ncm_fit_reset (NcmFit *fit);
 gboolean ncm_fit_run (NcmFit *fit, NcmFitRunMsgs mtype);
 
 gdouble ncm_fit_type_constrain_error (NcmFit *fit, gdouble p, gint nu, gdouble dir, NcmMSetFunc *func, gdouble z, gboolean walk);
-gdouble ncm_fit_function_error (NcmFit *fit, NcmMSetFunc *func, gdouble z, gboolean pretty_print);
+void ncm_fit_function_error (NcmFit *fit, NcmMSetFunc *func, gdouble *x, gboolean pretty_print, gdouble *f, gdouble *sigma_f);
 gdouble ncm_fit_function_cov (NcmFit *fit, NcmMSetFunc *func1, gdouble z1, NcmMSetFunc *func2, gdouble z2, gboolean pretty_print);
 
 #define NCM_FIT_NUMDIFF_SCALE (1.0e-4)
 #define NCM_FIT_NPARAM(fit) ((fit)->pt->nfree)
 #define NCM_FIT_DEFAULT_M2LNL_ABSTOL (0.0)
 #define NCM_FIT_DEFAULT_M2LNL_RELTOL (1e-13)
-#define NCM_FIT_MAXEVAL 10000
+#define NCM_FIT_MAXITER 10000
 
 G_END_DECLS
 
