@@ -203,6 +203,9 @@ _ncm_fit_gsl_mm_run (NcmFit *fit, NcmFitRunMsgs mtype)
   gint status;
   gdouble prec = NCM_FIT_DEFAULT_M2LNL_RELTOL;
 
+  if (ncm_fit_has_equality_constraints (fit) || ncm_fit_has_inequality_constraints (fit))
+    g_error ("_ncm_fit_gsl_mm_run: GSL algorithms do not support constraints.");
+
   ncm_mset_fparams_get_vector (fit->mset, fit->fstate->fparams);
   gsl_multimin_fdfminimizer_set (fit_gsl_mm->mm, &fit_gsl_mm->f, ncm_vector_gsl (fit->fstate->fparams), fit_gsl_mm->err_a, fit_gsl_mm->err_b);
 
@@ -229,7 +232,7 @@ _ncm_fit_gsl_mm_run (NcmFit *fit, NcmFitRunMsgs mtype)
       status = gsl_multimin_test_gradient (fit_gsl_mm->mm->gradient, pscale);
     ncm_fit_log_step (fit, fit_gsl_mm->mm->f);
   }
-  while ( (status == GSL_CONTINUE) && (fit->fstate->niter < fit->maxeval) );
+  while ( (status == GSL_CONTINUE) && (fit->fstate->niter < fit->maxiter) );
 
   fit->fstate->m2lnL = fit_gsl_mm->mm->f;
   fit->fstate->m2lnL_prec = gsl_blas_dnrm2 (fit_gsl_mm->mm->gradient) / fit_gsl_mm->mm->f;
