@@ -28,6 +28,7 @@
 
 #include <glib.h>
 #include <glib-object.h>
+#include <numcosmo/build_cfg.h>
 #include <numcosmo/math/ncm_fit.h>
 
 G_BEGIN_DECLS
@@ -69,6 +70,7 @@ struct _NcmLHRatio1d
   NcmFitRunMsgs mtype;
   NcmLHRatio1dRoot rtype;
   NcmMSetPIndex pi;
+  NcmMSetFunc *constraint;
   gdouble chisquare;
   gdouble lb;
   gdouble ub;
