@@ -37,6 +37,7 @@
 
 #include "math/ncm_model.h"
 #include "math/ncm_mset.h"
+#include "math/ncm_cfg.h"
 
 enum
 {
@@ -251,6 +252,7 @@ _ncm_model_constructed (GObject *object)
       g_array_index (model->vparam_pos, guint, i) = model->total_len;
       model->total_len += g_array_index (model->vparam_len, guint, i);
     }
+ 
     model->params = ncm_vector_new (model->total_len);
     model->p = model->params;
     g_array_set_size (model->ptypes, model->total_len);
@@ -353,7 +355,7 @@ _ncm_model_get_property (GObject *object, guint prop_id, GValue *value, GParamSp
       g_value_set_uint (value, model_class->vparam_len);
       break;
     case PROP_IMPLEMENTATION:
-      g_value_set_ulong (value, model_class->impl);
+      g_value_set_uint64 (value, model_class->impl);
       break;
     case PROP_REPARAM:
       g_value_set_object (value, model->reparam);
@@ -370,7 +372,12 @@ _ncm_model_get_property (GObject *object, guint prop_id, GValue *value, GParamSp
 static NcmModel *
 _ncm_model_copy (NcmModel *model)
 {
-  NcmModel *model_copy = g_object_new (G_OBJECT_TYPE (model), NULL);
+  gchar *obj_ser = ncm_cfg_serialize_to_string (G_OBJECT (model), TRUE);
+  NcmModel *model_copy = NCM_MODEL (ncm_cfg_create_from_string (obj_ser));
+  g_free (obj_ser);
+
+  /* Old method */
+  /* NcmModel *model_copy = g_object_new (G_OBJECT_TYPE (model), NULL); */
 
   if (ncm_model_peek_reparam (model))
   {
@@ -465,12 +472,12 @@ ncm_model_class_init (NcmModelClass *klass)
                                                         G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
   g_object_class_install_property (object_class,
                                    PROP_IMPLEMENTATION,
-                                   g_param_spec_ulong  ("implementation",
-                                                        NULL,
-                                                        "Bitwise specification of functions implementation",
-                                                        0, G_MAXULONG, 0,
-                                                        G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
-
+                                   g_param_spec_uint64  ("implementation",
+                                                         NULL,
+                                                         "Bitwise specification of functions implementation",
+                                                         0, G_MAXUINT64, 0,
+                                                         G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  
   g_object_class_install_property (object_class,
                                    PROP_PTYPES,
                                    g_param_spec_boxed  ("params-types",
@@ -488,28 +495,6 @@ ncm_model_class_init (NcmModelClass *klass)
                                                          G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
 }
 
-G_LOCK_DEFINE_STATIC (last_model_id);
-
-void
-ncm_model_class_register_id (NcmModelClass *model_class)
-{
-  if (model_class->model_id < 0)
-  {
-    static NcmModelID last_model_id = 0;
-    G_LOCK (last_model_id);
-    model_class->model_id = last_model_id++;
-    G_UNLOCK (last_model_id);
-  }
-  else
-  {
-    g_error ("This model or its parent is already registred, id = %d. This function must be use once and only in the defining model.", model_class->model_id);
-  }
-  if (model_class->model_id > NCM_MODEL_MAX_ID)
-    g_error ("Max model id was already attained. Increase by altering NCM_MODEL_MAX_ID.");
-
-  return;
-}
-
 /**
  * ncm_model_class_get_property: (skip)
  * @object: a #NcmModelClass.
@@ -753,8 +738,8 @@ ncm_model_class_add_params (NcmModelClass *model_class, guint sparam_len, guint
 void
 ncm_model_class_set_name_nick (NcmModelClass *model_class, const gchar *name, const gchar *nick)
 {
-  g_free (model_class->name);
-  g_free (model_class->nick);
+/*  g_free (model_class->name); 
+  g_free (model_class->nick); */
 
   model_class->name = g_strdup (name);
   model_class->nick = g_strdup (nick);
@@ -882,7 +867,7 @@ ncm_model_class_set_vparam (NcmModelClass *model_class, guint vparam_id, guint d
 void
 ncm_model_class_check_params_info (NcmModelClass *model_class)
 {
-  gulong i;
+  guint64 i;
   guint total_params_len = model_class->sparam_len + model_class->vparam_len;
   if (!total_params_len)
   {
