@@ -162,7 +162,7 @@ static void
 _nc_data_hubble_mean_func (NcmDataGaussDiag *diag, NcmMSet *mset, NcmVector *vp)
 {
   NcDataHubble *hubble = NC_DATA_HUBBLE (diag);
-  NcHICosmo *cosmo = NC_HICOSMO (ncm_mset_peek (mset, NC_HICOSMO_ID));
+  NcHICosmo *cosmo = NC_HICOSMO (ncm_mset_peek (mset, nc_hicosmo_id ()));
   
   gint i;
 
@@ -174,11 +174,17 @@ _nc_data_hubble_mean_func (NcmDataGaussDiag *diag, NcmMSet *mset, NcmVector *vp)
   }
 }
 
+#ifdef NUMCOSMO_HAVE_SQLITE3
 static gchar *_nc_data_hubble_function_query[] =
 {
-  "L. Verde sample", "SELECT z,p,s FROM kinematics WHERE param='H' ORDER BY z",
-  "Cabre sample", "SELECT z,p,s FROM kinematics WHERE param='H_CABRE' ORDER BY z"
+  "Simon 2005 H(z) sample", "SELECT z,p,s FROM kinematics WHERE param='Hz_Simon2005' ORDER BY z",
+  "Cabre sample", "SELECT z,p,s FROM kinematics WHERE param='H_CABRE' ORDER BY z",
+  "Stern 2009 H(z) sample", "SELECT z,p,s FROM kinematics WHERE param='Hz_Stern2009' ORDER BY z",
+  "Moresco 2012 H(z) BC03  sample", "SELECT z,p,s FROM kinematics WHERE param='BC03_Moresco2012' ORDER BY z",
+  "Moresco 2012 H(z) MaStro sample", "SELECT z,p,s FROM kinematics WHERE param='MaStro_Moresco2012' ORDER BY z",
+  "Busca 2013 H(z) BAO+WMAP sample", "SELECT z,p,s FROM kinematics WHERE param='BAO+WMAP_Busca2013' ORDER BY z",
 };
+#endif
 
 /**
  * nc_data_hubble_new:
@@ -252,12 +258,12 @@ nc_data_hubble_get_size (NcDataHubble *hubble)
 void
 nc_data_hubble_set_sample (NcDataHubble *hubble, NcDataHubbleId id)
 {
+#ifdef NUMCOSMO_HAVE_SQLITE3
   NcmData *data = NCM_DATA (hubble);
   NcmDataGaussDiag *diag = NCM_DATA_GAUSS_DIAG (hubble);
   
   g_assert (id < NC_DATA_HUBBLE_NSAMPLES);
 
-#ifdef NUMCOSMO_HAVE_SQLITE3
   {
     const gchar *query = _nc_data_hubble_function_query[id * 2 + 1];
     gint i, nrow, qncol, ret;
