@@ -25,52 +25,50 @@
 /**
  * SECTION:nc_distance
  * @title: Cosmological Distances and Times
- * @short_description: Calculates the cosmological distances and related quantities.
+ * @short_description: Calculate cosmological distances and related quantities.
  *
  * This object implements several distances used in cosmology, here we have
  * the following definitions.
  * 
- * The Hubble scale is simply defined as the inverse of the Hubble function
+ * The Hubble scale is simply defined as the inverse of the Hubble function $H(z)$ [nc_hicosmo_H()],
  * \begin{equation}\label{eq:def:DH}
  * D_H(z) = \frac{c}{H(z)}, \qquad D_{H0} = \frac{c}{H_0}. 
  * \end{equation}
- * where $c$ is the speed of light [ncm_c_c()], $H(z)$ is the Hubble function 
- * and $H_0 \equiv H(0)$ is the Hubble parameter. The comoving distance $D_c$ is 
+ * where $c$ is the speed of light [ncm_c_c()], $z$ is the redshift
+ * and $H_0 \equiv H(0)$ is the Hubble parameter [nc_hicosmo_H0()]. The comoving distance $D_c$ is 
  * defined as
  * \begin{equation}\label{eq:def:Dc}
- * D_c(z) = \int_0^z\frac{1}{E},
+ * D_c(z) = \int_0^z \frac{dz^\prime}{E (z^\prime)},
  * \end{equation}
- * where $z$ is the redshift and $E$ is the Hubble function over the Hubble 
- * parameter, i.e., $$E \equiv \frac{H(z)}{H_0}.$$ Note that both quantities are 
+ * where $E(z)$ is the normalized Hubble function [nc_hicosmo_E()], i.e., 
+ * \begin{equation}\label{eq:def:Ez} 
+ * E(z) \equiv \frac{H(z)}{H_0}.
+ * \end{equation}  
+ * Note that both quantities are 
  * adimensional, in other words, one can think them as in units of the Hubble 
  * scale today.
  * 
- * The curvature distance is defined as
- * \begin{equation}\label{eq:def:Dk}
- * D_k(z) = \sqrt{\vert\Omega_{k0}\vert}D_c(z),
- * \end{equation}
- * where $\Omega_{k0}$ is the value of the curvature today [nc_hicosmo_Omega_k()].
- * 
- * The transverse distance $D_t$ and its derivative with respect to $z$ are 
+ * The transverse comoving distance $D_t$ and its derivative with respect to $z$ are 
  * given by
  * \begin{equation}\label{eq:def:Dt}
  * D_t(z) = \frac{\sinh\left(\sqrt{\Omega_{k0}}D_c(z)\right)}{\sqrt{\Omega_{k0}}}, 
- * \qquad \frac{dD_t}{dz}(z) = \frac{\cosh\left(\sqrt{\Omega_{k0}}D_c(z)\right)}{E(z)}.
+ * \qquad \frac{dD_t}{dz}(z) = \frac{\cosh\left(\sqrt{\Omega_{k0}}D_c(z)\right)}{E(z)},
  * \end{equation}
- * Using the definition above we have the luminosity distance simply
+ * where $\Omega_{k0}$ is the value of the curvature today [nc_hicosmo_Omega_k()].
+ * Using the definition above we have that the luminosity distance is
  * \begin{equation}\label{eq:def:Dl}
- * D_l = (1+z)D_t(z).
+ * D_l = (1+z)D_t(z),
  * \end{equation}
- * Then, the distance modulus is defined by the following expression
+ * and the distance modulus is given by
  * \begin{equation}\label{eq:def:mu}
  * \mu(z) = 5\log_{10}(D_l(z)) + 25,
  * \end{equation}
  * where $\log_{10}$ represents the logarithm in the decimal base. Note that 
  * the distance modulus is usually defined as 
  * $$5\log_{10}(D_{H0}D_l(z)/\text{pc}) - 5,$$
- * where $\text{pc}$ is parsec [ncm_c_pc()], thus, this differs from our 
+ * where $\text{pc}$ is parsec [ncm_c_pc()]. Thus, this differs from our 
  * definition by a factor of $5\log_{10}(D_{H0}/\text{Mpc})$, where $\text{Mpc}$ 
- * is mega parsec [ncm_c_Mpc()].
+ * is megaparsec [ncm_c_Mpc()].
  * 
  * 
  */
@@ -83,6 +81,7 @@
 #include "nc_distance.h"
 #include "math/integral.h"
 #include "math/ncm_c.h"
+#include "math/ncm_cfg.h"
 #include "math/ncm_spline_cubic_notaknot.h"
 
 typedef struct _ComovingDistanceArgument{
@@ -314,7 +313,7 @@ nc_distance_init (NcDistance *dist)
  * @cosmo: a #NcHICosmo.
  *
  * Calculate the curvature scale today as defined in Eq $\eqref{eq:def:DH}$ in
- * units of mega parsec $\text{Mpc}$ [ncm_c_Mpc()].
+ * units of megaparsec (Mpc) [ncm_c_Mpc()].
  *
  * Returns: $D_{H0}$.
  */
@@ -330,9 +329,9 @@ static gdouble comoving_distance_integral_argument(gdouble z, gpointer p);
  * nc_distance_comoving:
  * @dist: a #NcDistance.
  * @cosmo: a #NcHICosmo.
- * @z: the redshift $z$.
+ * @z: redshift $z$.
  * 
- * Calculates the comoving distance $D_c$ as defined in Eq. $\eqref{eq:def:Dc}$.
+ * Calculate the comoving distance $D_c (z)$ as defined in Eq. $\eqref{eq:def:Dc}$.
  *
  * Returns: $D_c(z)$.
  */
@@ -379,32 +378,15 @@ dcddz (gdouble cd, gdouble z, gpointer userdata)
   return 1.0 / sqrt (E2);
 }
 
-/**
- * nc_distance_curvature:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
- * @z: the redshift $z$.
- *
- * Calculate the curvature distance $D_k$ defined in Eq. $\eqref{eq:def:Dk}$.
- *
- * Returns: $D_k(z)$.
- */
-gdouble
-nc_distance_curvature (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
-{
-  const gdouble sqrt_Omega_k = sqrt(fabs(nc_hicosmo_Omega_k (cosmo)));
-  return sqrt_Omega_k * nc_distance_comoving (dist, cosmo, z);
-}
-
 /**
  * nc_distance_transverse:
  * @dist: a #NcDistance.
  * @cosmo: a #NcHICosmo.
- * @z: the redshift $z$.
+ * @z: redshift $z$.
  *
- * Calculates the transverse distance $D_t$ defined in Eq. $\eqref{eq:def:Dt}$.
+ * Compute the transverse comoving distance $D_t (z)$ defined in Eq. $\eqref{eq:def:Dt}$.
  *
- * Returns: $D_T(z)$.
+ * Returns: $D_t(z)$.
  */
 gdouble
 nc_distance_transverse (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
@@ -444,9 +426,9 @@ nc_distance_transverse (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
  * nc_distance_dtransverse_dz:
  * @dist: a #NcDistance.
  * @cosmo: a #NcHICosmo.
- * @z: the redshift $z$.
+ * @z: redshift $z$.
  *
- * Calculates the derivative of $D_t$ with respect to $z$ defined in 
+ * Compute the derivative of $D_t(z)$ with respect to $z$ defined in 
  * Eq. $\eqref{eq:def:Dt}$.
  *
  * Returns: $\frac{dD_t(z)}{dz}$.
@@ -487,9 +469,9 @@ nc_distance_dtransverse_dz (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
  * nc_distance_luminosity:
  * @dist: a #NcDistance.
  * @cosmo: a #NcHICosmo.
- * @z: the redshift $z$.
+ * @z: redshift $z$.
  *
- * Calculates the luminosity distance $D_l$  defined in Eq. $\eqref{eq:def:Dl}$.
+ * Compute the luminosity distance $D_l(z)$  defined in Eq. $\eqref{eq:def:Dl}$.
  *
  * Returns: $D_l(z)$.
  */
@@ -506,9 +488,9 @@ nc_distance_luminosity (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
  * nc_distance_modulus:
  * @dist: a #NcDistance.
  * @cosmo: a #NcHICosmo.
- * @z: the redshift $z$.
+ * @z: redshift $z$.
  *
- * Calculates the distance modulus $\mu$ defined in Eq. $\eqref{eq:def:mu}$.
+ * Compute the distance modulus $\mu(z)$ defined in Eq. $\eqref{eq:def:mu}$.
  *
  * Returns: $\mu(z)$.
  */
@@ -525,10 +507,10 @@ nc_distance_modulus (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
  * nc_distance_luminosity_hef:
  * @dist: a #NcDistance.
  * @cosmo: a #NcHICosmo.
- * @z_he: the redshift $z_{he}$ in our local frame.
- * @z_cmb: the redshift $z_{CMB}$ in the CMB frame.
+ * @z_he: redshift $z_{he}$ in our local frame.
+ * @z_cmb: redshift $z_{CMB}$ in the CMB frame.
  *
- * Calculates the luminosity distance $D_l$ corrected to our local frame.
+ * Calculate the luminosity distance $D_l$ corrected to our local frame.
  *
  * Returns: $D_l(z_{hef},z_{CMB})$.
  */
@@ -545,10 +527,10 @@ nc_distance_luminosity_hef (NcDistance *dist, NcHICosmo *cosmo, gdouble z_he, gd
  * nc_distance_modulus_hef:
  * @dist: a #NcDistance.
  * @cosmo: a #NcHICosmo.
- * @z_he: the redshift $z_{he}$ in our local frame.
- * @z_cmb: the redshift $z_{CMB}$ in the CMB frame.
+ * @z_he: redshift $z_{he}$ in our local frame.
+ * @z_cmb: redshift $z_{CMB}$ in the CMB frame.
  *
- * Calculates the distance modulus using the frame corrected luminosity distance
+ * Calculate the distance modulus using the frame corrected luminosity distance
  * [nc_distance_luminosity_hef()].
  *
  * Returns: $\mu(z_{hef},z_{CMB})$.
@@ -567,9 +549,11 @@ nc_distance_modulus_hef (NcDistance *dist, NcHICosmo *cosmo, gdouble z_he, gdoub
  * @dist: a #NcDistance
  * @cosmo: a #NcHICosmo
  *
- * Calculate the angular diameter curvature scale $D_a$ defined in 
- * Eq. \eqref{eq:def:Da} at decoupling redshift $z_\star$ given by 
- * [nc_distance_decoupling_redshift()].
+ * We define the angular diameter curvature scale $D_a(z_\star)$ as
+ * $$D_a(z_\star) = \frac{E(z_\star)}{1 + z_\star} D_t(z_\star),$$ 
+ * where $z_\star$ is the decoupling redshift, given by [nc_distance_decoupling_redshift()], 
+ * $E(z_\star)$ is the normalized Hubble function [Eq. $\eqref{eq:def:Ez}$] and 
+ * $D_t(z_\star)$ is the transverse comoving distance [Eq. $\eqref{eq:def:Dt}$] both computed at $z_\star$.
  *
  * Returns: $D_a(z_\star)$.
  */
@@ -590,9 +574,16 @@ nc_distance_angular_diameter_curvature_scale (NcDistance *dist, NcHICosmo *cosmo
  * nc_distance_shift_parameter:
  * @dist: a #NcDistance.
  * @cosmo: a #NcHICosmo.
- * @z: the redshift $z$.
+ * @z: redshift $z$.
  *
- * Calculate the shift parameter $R$ defined in Eq. $\eqref{eq:def:R}$.
+ * The shift parameter $R(z)$ is defined as
+ * \begin{align}
+ * R(z) &=& \frac{\sqrt{\Omega_m H_0^2}}{c} (1 + z) D_A(z) \\
+ * &=& \sqrt{\Omega_m} D_t(z),
+ * \end{align}
+ * where $\Omega_m$ is the matter density paremeter [nc_hicosmo_Omega_m()],
+ * $D_A(z) = D_{H_0} D_t(z) / (1 + z)$ is the angular diameter distance and 
+ * $D_t(z)$ is the tranverse comoving distance [Eq. $\eqref{eq:def:Dt}$].
  *
  * Returns: $R(z)$.
  */
@@ -609,7 +600,7 @@ nc_distance_shift_parameter (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
  * @dist: a #NcDistance
  * @cosmo: a #NcHICosmo
  *
- * Calculate the shift parameter $R$ [nc_distance_shift_parameter()] at the
+ * Compute the shift parameter $R(z)$ [nc_distance_shift_parameter()] at the
  * decoupling redshift $z_\star$ [nc_distance_decoupling_redshift()].
  *
  * Returns: $R(z_\star)$.
@@ -628,55 +619,15 @@ nc_distance_shift_parameter_lss (NcDistance *dist, NcHICosmo *cosmo)
     return GSL_NAN;
 }
 
-/**
- * nc_distance_comoving_a0:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
- * @z: the redshift $z$.
- *
- * FIXME
- *
- * Returns: FIXME
- */
-gdouble
-nc_distance_comoving_a0 (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
-{
-  const gdouble sqrt_mod_Omega_k = sqrt(fabs(nc_hicosmo_Omega_k (cosmo)));
-  const gdouble comoving = nc_distance_comoving (dist, cosmo, z);
-  return sqrt_mod_Omega_k * comoving;
-}
-
-/**
- * nc_distance_comoving_a0_lss:
- * @dist: a #NcDistance.
- * @cosmo: a #NcHICosmo.
- *
- * Calculate the d_c (z_lss) / a0
- *
- * Returns: FIXME
- */
-gdouble
-nc_distance_comoving_a0_lss (NcDistance *dist, NcHICosmo *cosmo)
-{
-  const gdouble sqrt_mod_Omega_k = sqrt(fabs(nc_hicosmo_Omega_k (cosmo)));
-  const gdouble z_star = nc_distance_decoupling_redshift (dist, cosmo);
-  if (gsl_finite (z_star))
-  {
-    const gdouble comoving = nc_distance_comoving (dist, cosmo, z_star);
-    return sqrt_mod_Omega_k * comoving;
-  }
-  else
-    return GSL_NAN;
-}
-
 /**
  * nc_distance_comoving_lss:
  * @dist: a #NcDistance.
  * @cosmo: a #NcHICosmo.
  *
- * Calculate the d_c (z_lss)
+ * Compute the comoving distance $D_c(z)$ [Eq. \eqref{eq:def:Dc}] at the 
+ * decoupling redshift $z_\star$ [nc_distance_decoupling_redshift()].
  *
- * Returns: FIXME
+ * Returns: $D_c(z_\star)$.
  */
 gdouble
 nc_distance_comoving_lss (NcDistance *dist, NcHICosmo *cosmo)
@@ -693,9 +644,20 @@ nc_distance_comoving_lss (NcDistance *dist, NcHICosmo *cosmo)
  * @dist: a #NcDistance.
  * @cosmo: a #NcHICosmo.
  *
- * Decoupling redshift (arXiv:astro-ph/9510117).
+ * The decoupling redshift $z_\star$ corresponds to the epoch of 
+ * the last scattering surface of the cosmic microwave background photons. 
  *
- * Returns: FIXME
+ * This function computes $z_\star$ using [nc_hicosmo_z_lss()], if @cosmo implements 
+ * it, or using Hu & Sugiyama fitting formula <link linkend="XHu1996">Hu (1996)</link>, 
+ * $$ z_\star = 1048 \left(1 + 1.24 \times 10^{-3}  (\Omega_b h^2)^{-0.738}\right) \left(1 + g_1 (\Omega_m h^2)^{g_2}\right),$$
+ * where $\Omega_b h^2$ [nc_hicosmo_Omega_bh2()] and $\Omega_m h^2$ [nc_hicosmo_Omega_mh2()] 
+ * are, respectively, the baryonic and matter density parameters times the square 
+ * of the dimensionless Hubble parameter $h$,  $H_0 = 100 \, h \, \text{km/s} \, \text{Mpc}^{-1}$. 
+ * The parameters $g_1$ and $g_2$ are given by
+ * $$g_1 = \frac{0.0783 (\Omega_b h^2)^{-0.238}}{(1 + 39.5 (\Omega_b h^2)^{0.763})} 
+ * \; \text{and} \; g_2 = \frac{0.56}{\left(1 + 21.1 (\Omega_b h^2)^{1.81}\right)}.$$
+ * 
+ * Returns: $z_\star$
  */
 gdouble
 nc_distance_decoupling_redshift (NcDistance *dist, NcHICosmo *cosmo)
@@ -704,10 +666,8 @@ nc_distance_decoupling_redshift (NcDistance *dist, NcHICosmo *cosmo)
     return nc_hicosmo_z_lss (cosmo);
   else
   {
-    gdouble h = nc_hicosmo_h (cosmo);
-    gdouble h2 = h * h;
-    gdouble omega_b_h2 = nc_hicosmo_Omega_b (cosmo) * h2;
-    gdouble omega_m_h2 = nc_hicosmo_Omega_m (cosmo) * h2;
+    gdouble omega_b_h2 = nc_hicosmo_Omega_bh2 (cosmo);
+    gdouble omega_m_h2 = nc_hicosmo_Omega_mh2 (cosmo); 
     gdouble g1 = 0.0783 * pow (omega_b_h2, -0.238) / (1.0 + 39.5 * pow (omega_b_h2, 0.763));
     gdouble g2 = 0.560 / (1.0 + 21.1 * pow (omega_b_h2, 1.81));
     return 1048.0 * (1.0 + 1.24e-3 * pow (omega_b_h2, -0.738)) * (1.0 + g1 * pow (omega_m_h2, g2));
@@ -720,11 +680,23 @@ static gdouble sound_horizon_integral_argument(gdouble z, gpointer p);
  * nc_distance_sound_horizon:
  * @dist: a #NcDistance.
  * @cosmo: a #NcHICosmo.
- * @z: the redshift $z$.
+ * @z: redshift $z$.
  *
- * Calculate the sound horizon integrating numerically the hubble function.
+ * Compute the sound horizon $r_s$, 
+ * \begin{equation}
+ * r_s (z) = \int_{z}^\infty \frac{c_s(z^\prime)}{E(z^\prime)} dz^\prime,
+ * \end{equation}
+ * where $c_s(z)$ is the speed of sound wave and $E(z)$ is the normalized Hubble function [nc_hicosmo_E()].
+ * 
+ * The integrand is given by
+ * \begin{equation}\label{eq:def:rs:integrand}
+ * \frac{c_s(z^\prime)}{H(z^\prime)} = \frac{1}{\sqrt{E(z^\prime)^2 (3 + \frac{9}{4} (1 + 0.2271 n_{eff}) \frac{\Omega_b}{\Omega_r (1 + z^\prime)})}},
+ * \end{equation}
+ * where $n_{eff}$ is the effective number of neutrinos [ncm_c_neutrino_n_eff()], 
+ * $\Omega_b$ [nc_hicosmo_Omega_b()] and $\Omega_r$ [nc_hicosmo_Omega_r()] are the baryonic and radiation density parameter, respectively. 
+ * If $\Omega_r = 0$, the integrand returns 0.0.
  *
- * Returns: FIXME
+ * Returns: $r_s(z)$.
  */
 gdouble
 nc_distance_sound_horizon (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
@@ -762,11 +734,13 @@ sound_horizon_integral_argument (gdouble z, gpointer p)
  * nc_distance_dsound_horizon_dz:
  * @dist: a #NcDistance.
  * @cosmo: a #NcHICosmo.
- * @z: the redshift $z$.
+ * @z: redshift $z$.
  *
- * Calculate the sound horizon derivative.
+ * Calculate the sound horizon [nc_distance_sound_horizon()] derivative with respect to $z$, 
+ * $$\frac{d r_s(z)}{dz} = - \frac{c_s(z)}{E(z)},$$
+ * where $c_s(z) / E(z)$ is given by Eq. \eqref{eq:def:rs:integrand}.
  *
- * Returns: FIXME
+ * Returns: $\frac{d r_s(z)}{dz}$.
  */
 gdouble
 nc_distance_dsound_horizon_dz (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
@@ -779,9 +753,15 @@ nc_distance_dsound_horizon_dz (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
  * @dist: a #NcDistance.
  * @cosmo: a #NcHICosmo.
  *
- * Calculate the acoustic scale l_A = pi D_T (z_lss) / r (z_lss).
+ * Compute the acoustic scale $l_A (z_\star)$ at $z_\star$ [nc_distance_decoupling_redshift()],
+ * \begin{equation}
+ * l_A(z_\star) = \pi \frac{D_t (z_\star)}{r_s (z_\star)},
+ * \end{equation}
+ * where $D_t(z_\star)$ is the comoving transverse distance [nc_distance_transverse()] 
+ * and $r_s(z_\star)$ is the sound horizon [nc_distance_sound_horizon()] both 
+ * both computed at $z_\star$.
  *
- * Returns: FIXME
+ * Returns: $l_A(z_\star)$.
  */
 gdouble
 nc_distance_acoustic_scale (NcDistance *dist, NcHICosmo *cosmo)
@@ -798,17 +778,26 @@ nc_distance_acoustic_scale (NcDistance *dist, NcHICosmo *cosmo)
  * @dist: a #NcDistance.
  * @cosmo: a #NcHICosmo.
  *
- * Drag redshift (arXiv:astro-ph/9510117).
- *
- * Returns: FIXME
+ * Drag redshift is the epoch at which baryons were released from photons. 
+ * 
+ * This function computes $z_d$ using the fitting formula given in
+ * <link linkend="XEisenstein1998">Eisenstein & Hu (1998)</link>,
+ * $$z_d = \frac{1291 (\Omega_m h^2)^{0.251}}{(1 + 0.659 (\Omega_m h^2)^{0.828})}
+ * \left(1 + b_1 (\Omega_b h^2)^{b_2}\right),$$
+ * where $\Omega_b h^2$ [nc_hicosmo_Omega_bh2()] and $\Omega_m h^2$ [nc_hicosmo_Omega_mh2()] 
+ * are, respectively, the baryonic and matter density parameters times the square 
+ * of the dimensionless Hubble parameter $h$,  $H_0 = 100 \, h \, \text{km/s} \, \text{Mpc}^{-1}$. 
+ * The parameters $b_1$ and $b_2$ are given by
+ * $$b_1 = 0.313 (\Omega_m h^2)^{-0.419} \left(1 + 0.607 (\Omega_m h^2)^{0.674}\right) \; 
+ * \text{and} \; b_2 = 0.238 (\Omega_m h^2)^{0.223}.$$
+ * 
+ * Returns: $z_d$.
  */
 gdouble
 nc_distance_drag_redshift (NcDistance *dist, NcHICosmo *cosmo)
 {
-  gdouble h = nc_hicosmo_h (cosmo);
-  gdouble h2 = h*h;
-  gdouble omega_m_h2 = nc_hicosmo_Omega_m (cosmo) * h2;
-  gdouble omega_b_h2 = nc_hicosmo_Omega_b (cosmo) * h2;
+  gdouble omega_m_h2 = nc_hicosmo_Omega_mh2 (cosmo);
+  gdouble omega_b_h2 = nc_hicosmo_Omega_bh2 (cosmo);
   gdouble b1 = 0.313 * pow (omega_m_h2, -0.419) * (1.0 + 0.607 * pow (omega_m_h2, 0.674));
   gdouble b2 = 0.238 * pow (omega_m_h2, 0.223);
   return 1291.0 * pow (omega_m_h2, 0.251) / (1.0 + 0.659 * pow (omega_m_h2, 0.828)) *
@@ -819,11 +808,12 @@ nc_distance_drag_redshift (NcDistance *dist, NcHICosmo *cosmo)
  * nc_distance_dilation_scale:
  * @dist: a #NcDistance.
  * @cosmo: a #NcHICosmo.
- * @z: the redshift $z$.
+ * @z: redshift $z$.
  *
- * Dilation scale D_v(z) -- (arXiv:astro-ph/0501171)
+ * The dilation scale is the cube root of the product of the radial dilation
+ * times the square of the transverse dilation -- (arXiv:astro-ph/0501171)
  *
- * Returns: FIXME
+ * Returns: $D_V(z)$.
  */
 gdouble
 nc_distance_dilation_scale (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
@@ -840,7 +830,7 @@ nc_distance_dilation_scale (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
  * @cosmo: a #NcHICosmo.
  * @z: the redshift $z$.
  *
- * Bao 'A' scaleD_v(z) sqrt(Omega_m) / z -- (arXiv:astro-ph/0501171)
+ * Bao 'A' scale D_v(z) sqrt(Omega_m) / z -- (arXiv:astro-ph/0501171)
  *
  * Returns: FIXME
  */
@@ -865,14 +855,19 @@ nc_distance_bao_A_scale (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
 gdouble
 nc_distance_bao_r_Dv (NcDistance *dist, NcHICosmo *cosmo, gdouble z)
 {
-  gdouble zd = nc_distance_drag_redshift (dist, cosmo);
   gdouble r_zd;
   gdouble Dv;
 
-  if (!gsl_finite (zd))
-    return GSL_NAN;
+  if (ncm_model_impl (NCM_MODEL (cosmo)) & NC_HICOSMO_IMPL_as_drag)
+    r_zd = nc_hicosmo_as_drag (cosmo);
+  else
+  {
+    gdouble zd = nc_distance_drag_redshift (dist, cosmo);
+    if (!gsl_finite (zd))
+      return GSL_NAN;
+    r_zd = nc_distance_sound_horizon (dist, cosmo, zd);
+  }
 
-  r_zd = nc_distance_sound_horizon (dist, cosmo, zd);
   Dv = nc_distance_dilation_scale (dist, cosmo, z);
   return r_zd / Dv;
 }
@@ -979,14 +974,14 @@ static void
 _nc_distance_func0 (NcmMSet *mset, gpointer obj, const gdouble *x, gdouble *f)
 {
   NcDistanceFuncData *dist_data = (NcDistanceFuncData *)obj;
-  f[0] = dist_data->f0 (dist_data->dist, NC_HICOSMO (ncm_mset_peek (mset, NC_HICOSMO_ID)));
+  f[0] = dist_data->f0 (dist_data->dist, NC_HICOSMO (ncm_mset_peek (mset, nc_hicosmo_id ())));
 }
 
 static void
 _nc_distance_func1 (NcmMSet *mset, gpointer obj, const gdouble *x, gdouble *f)
 {
   NcDistanceFuncData *dist_data = (NcDistanceFuncData *)obj;
-  f[0] = dist_data->f1 (dist_data->dist, NC_HICOSMO (ncm_mset_peek (mset, NC_HICOSMO_ID)), x[0]);
+  f[0] = dist_data->f1 (dist_data->dist, NC_HICOSMO (ncm_mset_peek (mset, nc_hicosmo_id ())), x[0]);
 }
 
 static void
