@@ -27,6 +27,7 @@
 
 #include <glib.h>
 #include <glib-object.h>
+#include <numcosmo/build_cfg.h>
 #include <numcosmo/nc_hicosmo.h>
 #include <numcosmo/math/ncm_ode_spline.h>
 #include <numcosmo/math/ncm_model_ctrl.h>
@@ -88,17 +89,15 @@ gdouble nc_distance_decoupling_redshift (NcDistance *dist, NcHICosmo *cosmo);
 gdouble nc_distance_drag_redshift (NcDistance *dist, NcHICosmo *cosmo);
 gdouble nc_distance_shift_parameter_lss (NcDistance *dist, NcHICosmo *cosmo);
 gdouble nc_distance_comoving_lss (NcDistance *dist, NcHICosmo *cosmo);
-gdouble nc_distance_comoving_a0_lss (NcDistance *dist, NcHICosmo *cosmo);
 gdouble nc_distance_acoustic_scale (NcDistance *dist, NcHICosmo *cosmo);
 gdouble nc_distance_Omega_k (NcDistance *dist, NcHICosmo *cosmo);
+gdouble nc_distance_angular_diameter_curvature_scale (NcDistance *dist, NcHICosmo *cosmo);
 
 /***************************************************************************
  * Redshift dependent 'distances'
  ****************************************************************************/
 
 gdouble nc_distance_comoving (NcDistance *dist, NcHICosmo *cosmo, gdouble z);
-gdouble nc_distance_comoving_a0 (NcDistance *dist, NcHICosmo *cosmo, gdouble z);
-gdouble nc_distance_curvature (NcDistance *dist, NcHICosmo *cosmo, gdouble z);
 gdouble nc_distance_transverse (NcDistance *dist, NcHICosmo *cosmo, gdouble z);
 gdouble nc_distance_luminosity (NcDistance *dist, NcHICosmo *cosmo, gdouble z);
 gdouble nc_distance_modulus (NcDistance *dist, NcHICosmo *cosmo, gdouble z);
@@ -108,6 +107,7 @@ gdouble nc_distance_shift_parameter (NcDistance *dist, NcHICosmo *cosmo, gdouble
 gdouble nc_distance_dilation_scale (NcDistance *dist, NcHICosmo *cosmo, gdouble z);
 gdouble nc_distance_bao_A_scale (NcDistance *dist, NcHICosmo *cosmo, gdouble z);
 gdouble nc_distance_sound_horizon (NcDistance *dist, NcHICosmo *cosmo, gdouble z);
+gdouble nc_distance_dsound_horizon_dz (NcDistance *dist, NcHICosmo *cosmo, gdouble z);
 gdouble nc_distance_bao_r_Dv (NcDistance *dist, NcHICosmo *cosmo, gdouble z);
 
 /***************************************************************************
