@@ -27,6 +27,7 @@
 
 #include <glib.h>
 #include <glib-object.h>
+#include <numcosmo/build_cfg.h>
 #include <numcosmo/math/ncm_c.h>
 #include <numcosmo/math/ncm_model.h>
 #include <numcosmo/math/ncm_mset_func.h>
@@ -49,13 +50,14 @@ G_BEGIN_DECLS
  * @NC_HICOSMO_IMPL_Omega_c: FIXME
  * @NC_HICOSMO_IMPL_Omega_t: FIXME
  * @NC_HICOSMO_IMPL_sigma_8: FIXME
- * @NC_HICOSMO_IMPL_T_gamma0: FIXME
- * @NC_HICOSMO_IMPL_z_lss: FIXME
- * @NC_HICOSMO_IMPL_E2: FIXME
+ * @NC_HICOSMO_IMPL_T_gamma0: Radiation temperature today
+ * @NC_HICOSMO_IMPL_z_lss: Redshift of the last scatering surface
+ * @NC_HICOSMO_IMPL_as_drag: Acoustic Scale at drag redshift 
+ * @NC_HICOSMO_IMPL_E2: Adimensinal Hubble function squared 
  * @NC_HICOSMO_IMPL_dE2_dz: FIXME
  * @NC_HICOSMO_IMPL_d2E2_dz2: FIXME
- * @NC_HICOSMO_IMPL_cd: FIXME
- * @NC_HICOSMO_IMPL_powspec: FIXME
+ * @NC_HICOSMO_IMPL_cd: Comoving distance
+ * @NC_HICOSMO_IMPL_powspec: Perturbations power spectrum
  *
  * FIXME
  */
@@ -69,12 +71,13 @@ typedef enum _NcHICosmoImpl
   NC_HICOSMO_IMPL_sigma_8   = 1 << 5,
   NC_HICOSMO_IMPL_T_gamma0  = 1 << 6,
   NC_HICOSMO_IMPL_z_lss     = 1 << 7,
-  NC_HICOSMO_IMPL_E2        = 1 << 8,
-  NC_HICOSMO_IMPL_dE2_dz    = 1 << 9,
-  NC_HICOSMO_IMPL_d2E2_dz2  = 1 << 10,
-  NC_HICOSMO_IMPL_cd        = 1 << 11,
-  NC_HICOSMO_IMPL_powspec   = 1 << 12, /*< private >*/
-  NC_HICOSMO_IMPL_LAST      = 1 << 13, /*< skip >*/
+  NC_HICOSMO_IMPL_as_drag   = 1 << 8,
+  NC_HICOSMO_IMPL_E2        = 1 << 9,
+  NC_HICOSMO_IMPL_dE2_dz    = 1 << 10,
+  NC_HICOSMO_IMPL_d2E2_dz2  = 1 << 11,
+  NC_HICOSMO_IMPL_cd        = 1 << 12,
+  NC_HICOSMO_IMPL_powspec   = 1 << 13, /*< private >*/
+  NC_HICOSMO_IMPL_LAST      = 1 << 14, /*< skip >*/
 } NcHICosmoImpl;
 
 typedef struct _NcHICosmoClass NcHICosmoClass;
@@ -94,6 +97,7 @@ struct _NcHICosmoClass
   NcmModelFunc0 sigma_8;
   NcmModelFunc0 T_gamma0;
   NcmModelFunc0 z_lss;
+  NcmModelFunc0 as_drag;
   NcmModelFunc1 E2;
   NcmModelFunc1 dE2_dz;
   NcmModelFunc1 d2E2_dz2;
@@ -114,10 +118,10 @@ struct _NcHICosmo
   gboolean is_eternal;
 };
 
-extern gint32 NC_HICOSMO_ID;
-
 GType nc_hicosmo_get_type (void) G_GNUC_CONST;
 
+NCM_MSET_MODEL_DECLARE_ID (nc_hicosmo);
+
 /*
  * Cosmological model constant functions
  */
@@ -129,6 +133,7 @@ G_INLINE_FUNC gdouble nc_hicosmo_Omega_t (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_T_gamma0 (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_sigma_8 (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_z_lss (NcHICosmo *cosmo);
+G_INLINE_FUNC gdouble nc_hicosmo_as_drag (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_E2 (NcHICosmo *cosmo, gdouble x);
 G_INLINE_FUNC gdouble nc_hicosmo_dE2_dz (NcHICosmo *cosmo, gdouble x);
 G_INLINE_FUNC gdouble nc_hicosmo_d2E2_dz2 (NcHICosmo *cosmo, gdouble x);
@@ -139,13 +144,19 @@ G_INLINE_FUNC gdouble nc_hicosmo_Omega_k (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_Omega_m (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_h (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_h2 (NcHICosmo *cosmo);
+G_INLINE_FUNC gdouble nc_hicosmo_Omega_bh2 (NcHICosmo *cosmo);
+G_INLINE_FUNC gdouble nc_hicosmo_Omega_ch2 (NcHICosmo *cosmo);
+G_INLINE_FUNC gdouble nc_hicosmo_Omega_rh2 (NcHICosmo *cosmo);
+G_INLINE_FUNC gdouble nc_hicosmo_Omega_mh2 (NcHICosmo *cosmo);
 G_INLINE_FUNC gdouble nc_hicosmo_E (NcHICosmo *cosmo, gdouble z);
+G_INLINE_FUNC gdouble nc_hicosmo_Em2 (NcHICosmo *cosmo, gdouble z);
 G_INLINE_FUNC gdouble nc_hicosmo_H (NcHICosmo *cosmo, gdouble z);
 G_INLINE_FUNC gdouble nc_hicosmo_dH_dz (NcHICosmo *cosmo, gdouble z);
 G_INLINE_FUNC gdouble nc_hicosmo_j (NcHICosmo *cosmo, gdouble z);
 G_INLINE_FUNC gdouble nc_hicosmo_qp (NcHICosmo *cosmo, gdouble z);
 G_INLINE_FUNC gdouble nc_hicosmo_q (NcHICosmo *cosmo, gdouble z);
-G_INLINE_FUNC gint32 nc_hicosmo_id (void);
+G_INLINE_FUNC gdouble nc_hicosmo_dec (NcHICosmo *cosmo, gdouble z);
+G_INLINE_FUNC gdouble nc_hicosmo_wec (NcHICosmo *cosmo, gdouble z);
 
 NcHICosmo *nc_hicosmo_new_from_name (GType parent_type, gchar *cosmo_name);
 void nc_hicosmo_log_all_models (GType parent);
@@ -161,6 +172,8 @@ void nc_hicosmo_set_Omega_c_impl (NcHICosmoClass *model_class, NcmModelFunc0 f);
 void nc_hicosmo_set_Omega_t_impl (NcHICosmoClass *model_class, NcmModelFunc0 f);
 void nc_hicosmo_set_sigma_8_impl (NcHICosmoClass *model_class, NcmModelFunc0 f);
 void nc_hicosmo_set_T_gamma0_impl (NcHICosmoClass *model_class, NcmModelFunc0 f);
+void nc_hicosmo_set_z_lss_impl (NcHICosmoClass *model_class, NcmModelFunc0 f);
+void nc_hicosmo_set_as_drag_impl (NcHICosmoClass *model_class, NcmModelFunc0 f);
 
 void nc_hicosmo_set_E2_impl (NcHICosmoClass *model_class, NcmModelFunc1 f);
 void nc_hicosmo_set_dE2_dz_impl (NcHICosmoClass *model_class, NcmModelFunc1 f);
@@ -189,6 +202,7 @@ NCM_MODEL_FUNC0_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,Omega_t)
 NCM_MODEL_FUNC0_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,T_gamma0)
 NCM_MODEL_FUNC0_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,sigma_8)
 NCM_MODEL_FUNC0_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,z_lss)
+NCM_MODEL_FUNC0_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,as_drag)
 
 NCM_MODEL_FUNC1_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,E2)
 NCM_MODEL_FUNC1_IMPL (NC_HICOSMO,NcHICosmo,nc_hicosmo,dE2_dz)
@@ -232,10 +246,40 @@ nc_hicosmo_h2 (NcHICosmo *cosmo)
   return gsl_pow_2 (nc_hicosmo_H0 (cosmo) / 100.0);
 }
 
+G_INLINE_FUNC gdouble
+nc_hicosmo_Omega_bh2 (NcHICosmo *cosmo)
+{
+  return nc_hicosmo_h2 (cosmo) * nc_hicosmo_Omega_b (cosmo);
+}
+
+G_INLINE_FUNC gdouble
+nc_hicosmo_Omega_ch2 (NcHICosmo *cosmo)
+{
+  return nc_hicosmo_h2 (cosmo) * nc_hicosmo_Omega_c (cosmo);
+}
+
+G_INLINE_FUNC gdouble
+nc_hicosmo_Omega_rh2 (NcHICosmo *cosmo)
+{
+  return nc_hicosmo_h2 (cosmo) * nc_hicosmo_Omega_r (cosmo);
+}
+
+G_INLINE_FUNC gdouble
+nc_hicosmo_Omega_mh2 (NcHICosmo *cosmo)
+{
+  return nc_hicosmo_h2 (cosmo) * nc_hicosmo_Omega_m (cosmo);
+}
+
 G_INLINE_FUNC gdouble
 nc_hicosmo_E (NcHICosmo *cosmo, gdouble z)
 {
-  return sqrt(nc_hicosmo_E2 (cosmo, z));
+  return sqrt (nc_hicosmo_E2 (cosmo, z));
+}
+
+G_INLINE_FUNC gdouble
+nc_hicosmo_Em2 (NcHICosmo *cosmo, gdouble z)
+{
+  return 1.0 / nc_hicosmo_E2 (cosmo, z);
 }
 
 G_INLINE_FUNC gdouble
@@ -254,6 +298,29 @@ nc_hicosmo_q (NcHICosmo *cosmo, gdouble z)
   return (dE2_dz * (1.0 + z) / (2.0 * E2) - 1.0);
 }
 
+G_INLINE_FUNC gdouble
+nc_hicosmo_dec (NcHICosmo *cosmo, gdouble z)
+{
+  gdouble q, E2, Ok;
+  gdouble x = 1.0 + z;
+  q = nc_hicosmo_q (cosmo, z);
+  E2 = nc_hicosmo_E2 (cosmo, z);
+  Ok = nc_hicosmo_Omega_k (cosmo);
+  
+  return (2.0 * E2 - q * E2 - 2.0 * Ok * x * x) / 3.0;
+}
+
+G_INLINE_FUNC gdouble
+nc_hicosmo_wec (NcHICosmo *cosmo, gdouble z)
+{
+  gdouble E2, Ok;
+  gdouble x = 1.0 + z;
+  E2 = nc_hicosmo_E2 (cosmo, z);
+  Ok = nc_hicosmo_Omega_k (cosmo);
+  
+  return (E2 - Ok * x * x);
+}
+
 G_INLINE_FUNC gdouble
 nc_hicosmo_qp (NcHICosmo *cosmo, gdouble z)
 {
@@ -276,12 +343,6 @@ nc_hicosmo_j (NcHICosmo *cosmo, gdouble z)
   return gsl_pow_2 (1.0 + z) * (d2E2_dz2 - 2.0 * dE2_dz / (1.0 + z)) / (2.0 * E2) + 1.0;
 }
 
-G_INLINE_FUNC gint32
-nc_hicosmo_id (void)
-{
-  return NC_HICOSMO_ID;
-}
-
 G_END_DECLS
 
 #endif /* NUMCOSMO_HAVE_INLINE */
